import React from 'react';
import {
    Modal,
    ModalContent,
    ModalHeader,
    ModalBody,
    Button,
    useDisclosure,
    Listbox,
    ListboxItem,
    Select,
    SelectItem,
} from '@nextui-org/react';
import { IStationData } from 'app/services/api/interfaces.server';
import { useSearchParams, useNavigate } from '@remix-run/react';

type Tdb = {
    stationsData: IStationData[];
    ownersList: string[];
    wagonTypeList: string[][];
};

type Tfilters = {
    [filterKey: string]: {
        title: string;
        items: { key: string; value: string }[];
    };
};

const filterMenuKeys: Record<string, string> = {
    wagons: 'Вагоны',
    ways: 'Ж/Д Пути',
    stations: 'Станции',
};

function getFilterSelectedKeys(searchParameters: URLSearchParams) {
    const result: {
        [filterMenuKey: string]: { [filterKey: string]: Set<string> };
    } = {};
    const _filters = [...searchParameters.entries()]
        .filter((parameter) => parameter[0].startsWith('filter'))
        .map((parameter) => [parameter[0].replace('filter', ''), parameter[1]]);
    for (const [name, value] of _filters) {
        const [filterMenuKey, filterKey] = name
            .match(/[A-Z][a-z]*/g)!
            .map((value_) => value_.toLowerCase());
        if (!(filterMenuKey in result)) {
            result[filterMenuKey] = {};
        }
        result[filterMenuKey][filterKey] = new Set(value.split(','));
    }
    return result;
}

export function FilterMenu({
    activeFilterMenuKey,
    filters,
    statesSelectedKeys,
}: {
    activeFilterMenuKey: string;
    filters: Tfilters;
    statesSelectedKeys: {
        [activeFilterMenuKey: string]: {
            [filterKey: string]: [
                Set<string>,
                React.Dispatch<React.SetStateAction<Set<string>>>,
            ];
        };
    };
}) {
    const [searchParameters] = useSearchParams();

    const _t = getFilterSelectedKeys(searchParameters);
    for (const filterKey of Object.keys(filters)) {
        if (
            !(
                filterKey in
                Object.keys(statesSelectedKeys[activeFilterMenuKey])
            ) &&
            !(activeFilterMenuKey in _t)
        ) {
            _t[activeFilterMenuKey] = {};
        }
    }

    return (
        <>
            {Object.keys(filters).map((filterKey) => {
                return (
                    <>
                        <h4>{filters[filterKey].title}</h4>
                        <Select
                            aria-label="Один из пунктов фильтра"
                            placeholder="Не выбрано"
                            selectionMode="multiple"
                            onChange={(event)=> {
                                statesSelectedKeys[activeFilterMenuKey][
                                    filterKey
                                ][1](new Set(event.target.value.split(",")))
                            }}
                            selectedKeys={
                                statesSelectedKeys[activeFilterMenuKey][
                                    filterKey
                                ][0]
                            }
                            onSelectionChange={(keys) =>
                               (keys as Set<string>)
                            }
                        >
                            {filters[filterKey].items.map((value) => {
                                return (
                                    <SelectItem
                                        key={value.key}
                                        value={value.value}
                                    >
                                        {value.key}
                                    </SelectItem>
                                );
                            })}
                        </Select>
                    </>
                );
            })}
        </>
    );
}

export default function ModalFilter({ db }: { db: Tdb }) {
    const filters: { [filterMenuKey: string]: Tfilters } = {
        wagons: {
            owner: {
                title: 'Собственник',
                items: db.ownersList.map((owner) => {
                    return { key: owner, value: owner };
                }),
            },
            state: {
                title: 'Состояние вагона',
                items: [
                    {
                        key: 'Не прибыл',
                        value: 'NotArrived',
                    },
                    {
                        key: 'Простаивает',
                        value: 'StandingBy',
                    },
                    {
                        key: 'Закончил',
                        value: 'Finished',
                    },
                    {
                        key: 'В работе',
                        value: 'inProgress',
                    },
                ],
            },
            type: {
                title: 'Тип вагона',
                items: db.wagonTypeList.map((type) => {
                    return { key: type[1], value: type[0] };
                }),
            },
        },
        ways: {
            empty: {
                title: 'Загруженность пути',
                items: [
                    {
                        key: 'Полная',
                        value: 'full',
                    },
                    {
                        key: 'Частичная',
                        value: 'half',
                    },
                    {
                        key: 'Отсутствует',
                        value: 'empty',
                    },
                ],
            },
        },
    };
    const [searchParameters] = useSearchParams();
    const navigate = useNavigate();
    const { isOpen, onOpen, onOpenChange } = useDisclosure();
    const [activeFilterMenuKey, setActiveFilterMenuKey] = React.useState(
        Object.keys(filterMenuKeys)[0],
    );
    const statesSelectedKeys: {
        [activeFilterMenuKey: string]: {
            [filterKey: string]: [
                Set<string>,
                React.Dispatch<React.SetStateAction<Set<string>>>,
            ];
        };
    } = {};
    statesSelectedKeys[activeFilterMenuKey] = {};
    const _t = getFilterSelectedKeys(searchParameters);

    for (const filterMenuKey in filters) {
        if (!(filterMenuKey in statesSelectedKeys)) {
            statesSelectedKeys[filterMenuKey] = {};
        }
        for (const filterKey in filters[filterMenuKey]) {
            /* eslint-disable-next-line */
            statesSelectedKeys[filterMenuKey][filterKey] = React.useState(
                _t[filterMenuKey] === undefined
                    ? new Set()
                    : _t[filterMenuKey][filterKey] ?? new Set(),
            );
        }
    }

    return (
        <div className="mb-4">
            <Button onPress={onOpen}>Фильтр</Button>
            <Modal
                isOpen={isOpen}
                onOpenChange={onOpenChange}
                scrollBehavior="outside"
                size="xl"
            >
                <ModalContent aria-label="Пользовательский фильтр">
                    {(onClose) => (
                        <>
                            <ModalHeader>Пользовательский фильтр</ModalHeader>
                            <ModalBody>
                                <Listbox
                                    onAction={(menuKey) =>
                                        setActiveFilterMenuKey(
                                            menuKey.toString(),
                                        )
                                    }
                                >
                                    {Object.keys(filterMenuKeys).map((key) => (
                                        <ListboxItem key={key}>
                                            {filterMenuKeys[key]}
                                        </ListboxItem>
                                    ))}
                                </Listbox>
                                <div>
                                    <FilterMenu
                                        activeFilterMenuKey={
                                            activeFilterMenuKey
                                        }
                                        filters={filters[activeFilterMenuKey]}
                                        statesSelectedKeys={statesSelectedKeys}
                                    />
                                </div>
                                <Button
                                    onClick={() => {
                                        for (const menuKey in statesSelectedKeys) {
                                            for (const filterKey in statesSelectedKeys[
                                                menuKey
                                            ]) {
                                                const _m=Array.from(
                                                    statesSelectedKeys[
                                                        menuKey
                                                    ][
                                                        filterKey
                                                    ][0].values()
                                                )
                                                if (
                                                    _m[0] === '' || _m.length==0
                                                ) {
                                                    searchParameters.delete(
                                                        `filter${
                                                            menuKey[0].toUpperCase() +
                                                            menuKey.slice(1)
                                                        }${
                                                            filterKey[0].toUpperCase() +
                                                            filterKey.slice(1)
                                                        }`,
                                                    );
                                                } else {
                                                    searchParameters.set(
                                                        `filter${
                                                            menuKey[0].toUpperCase() +
                                                            menuKey.slice(1)
                                                        }${
                                                            filterKey[0].toUpperCase() +
                                                            filterKey.slice(1)
                                                        }`,
                                                        _m.join(','),
                                                    );
                                                }
                                            }
                                        }
                                        navigate(
                                            '?' + searchParameters.toString(),
                                        );
                                        onClose();
                                    }}
                                >
                                    Применить
                                </Button>
                                <Button onClick={onClose}>Отмена</Button>
                            </ModalBody>
                        </>
                    )}
                </ModalContent>
            </Modal>
        </div>
    );
}
